//** ******************************************************************** ** ******************************************************************** **//
//**                                                                      **                                                                      **//
//**                    ** English **                                     **                     ** Italiano **                                   **//
//**                                                                      **                                                                      **//
//** ******************************************************************** ** ******************************************************************** **//
//**                                                                      **                                                                      **//
//** Sketch name......: CorrOpenGateWiFi                                  ** Nome Sketch.......: CorrOpenGateWiFi                                 **//
//** Author...........: Corrado Bevilacqua                                ** Autore............: Corrado Bevilacqua                               **//
//**                    http:\\myelectronichome.altervista.org            **                     http:\\myelectronichome.altervista.org           **//
//** Short description: Open three passages (Gate, small gate, garage)    ** Descrizione breve.: Apre tre varchi (cancello, Garage Cancelletto)   **//
//**                    via WiFi from smartphone or other devices         **                     via WiFi da un cellulare o altro dispositivo.    **//
//** ******************************************************************** ** ******************************************************************** **//
//** Open gate WiFi with Wemos D1                                         ** Apricancello WiFi con Wemos D1                                       **//
//** -------------------------------------------------------------------- ** -------------------------------------------------------------------- **//
//** This scketch allows us to open 3 doors from a mobile phone via WiFi. ** Questo scketch permette di aprire  3  varchi da un cellulare tramite **//
//** The passages are referred to as "garage","small gate" and "gate";    ** il  WiFi.  I   varchi sono  indicati  come  "garage",  "cancelletto" **//
//** but these descriptions are freely changeable.                        ** e "cancello"  ma   tali  descrizioni  sono  liberamente  cambiabili. **//
//** Use a Wemos D1 card connected to a 4 relay card.                     ** Utilizza  una  scheda  Wemos D1  collegata  ad  una scheda a 4 relè. **//
//**                                                                      **                                                                      **//
//** OPERATION :                                                          ** FUNZIONAMENTO:                                                       **//
//** Once powered, the  Wemos card behaves  like  an "access point" and a ** Una volta alimentata, la scheda si comporta come un "access point" e **//
//** "web server". Its SSID and password are visible in the lines:        ** "server web". Il suo SSID  e la  password sono visibili nelle righe: **//
//** const char* ssid = "WemosD1"  / const char* password = "WemosD1Key"; ** const char* ssid = "WemosD1"  / const char* password = "WemosD1Key"; **//
//** Once  the  mobile phone is connected to WiFi, you can go to the page ** Una volta collegato il cellulare al WiFi, si  può andare alla pagina **//
//** http:\\1.2.3.4 from your browser.  The address 1.2.3.4 is modifiable ** http:\\1.2.3.4 dal vostro browser. L'indirizzo 1.2.3.4 è modificabi- **//
//** in the line: IPAddress apIP (1, 2, 3, 4);.  At the password request, ** le nella riga: IPAddress apIP(1, 2, 3, 4);. Alla richiesta password, **//
//** log in using the "Admin" user and password "12345".  Also these  can ** autenticatevi tramite l'utenza  "Admin" e  password "12345".   Anche **//
//** be modified by acting on the IF containing:                          ** queste modificabili agendo sull'IF che contiene:                     **//
//** (server.arg("USERNAME")=="Admin" && server.arg("PASSWORD")=="12345") ** (server.arg("USERNAME")=="Admin" && server.arg("PASSWORD")=="12345") **//
//** Once authentication is complete, you will enter in the web page with ** Superata l'autenticazione entrerete nella pagina web con i pulsanti. **//
//** the buttons. The touch of a button generates a pulse on the          ** Il tocco di un pulsante genera un impulso sul corrispondente  pin al **//
//** corresponding pin, to which a relay that controls the electric lock  ** quale  è  collegato  un relè che comanda  la serratura elettrica che **//
//** is connected and allows the opening of the passage.                  ** permette l'apertura del varco.                                       **//
//**                                                                      **                                                                      **//
//** SECURITY                                                             ** SICUREZZA                                                            **//
//** To increase security, despite WPA2 and the  authentication page,  it ** Per incrementare la sicurezza, nonostante  la crittografia WPA2 e il **//
//** is  expected  that  the card  is always off. It is activated only by ** livello di autenticazione, è  previsto  che  la  scheda  sia  sempre **//
//** pressing  a hidden button  and immediately  it turns off  under  the ** spenta. Si  attiva  solo  alla  pressione  di un pulsante nascosto e **//
//** following conditions:                                                ** immediatamente si  autospegne sotto le seguenti condizioni:          **//
//**    1) after 120 seconds of inactivity;                               **    1) dopo  120 secondi di inattività;                               **//
//**    2) regardless of inactivity, however after 300 seconds            **    2) indipendentemente dall'inattività, comunque  dopo  300 secondi **//
//**       (5 minutes) from switching on the Wemos card;                  **       (5 minuti) dall'accensione della scheda Wemos;                 **//
//**    3) consequently to three failed authentication attempts;          **    3) conseguentemente a tre tentativi di autenticazione falliti;    **//
//**                                                                      **                                                                      **//
//** AUTO SWITCH-OFF                                                      ** AUTOSPEGNIMENTO:                                                     **//
//** Auto switch-off is only possible by connecting the contacts of a     ** L'autospegnimento è possibile solo collegando i contatti di  un relè **//
//** relay in parallel with the activation button. By pressing this       ** in parallelo al pulsante di attivazione. Premendo tale  pulsante, si **//
//** button, the Wemos card is powered. This card immediately activates   ** alimenta la scheda Wemos. Questa immediatamente attiva il relè con i **//
//** the relay that, in this manner, keeps the power supply active even   ** contatti   in   parallelo  al  pulsante  stesso  e  quindi  mantiene **//
//** after the release of the button.                                     ** l'alimentazione  attiva  anche  dopo  il  rilascio del pulsante.  In **//
//** So the power supply is under the control of a relay, controlled by   ** questo modo l'alimentazione è sotto il controllo di  un relè  a  sua **//
//** the Wemos card.                                                      ** volta comandato dalla scheda Wemos.                                  **//
//**                                                                      **                                                                      **//
//** NECESSARY MATERIAL:                                                  ** MATERIALE NECESSARIO:                                                **//
//** 1) Wemos D1 card. (It works like an Arduino board.It has a limited   ** 1) Scheda  Wemos D1. (Funziona come una scheda Arduino. Ha un numero **//
//** number of usable pins but it has WiFi on board).                     **    limitato di pin utilizzabili ma ha il WiFi a bordo).              **//
//** 2) A piece of wire, to be used as an antenna. To be connected to the ** 2) Uno spezzone di filo, da utilizzare come antenna. Da collegare al **//
//**    analog pin 0. In this way the detectable value is almost random   **    pin analogico 0. In questo modo il valore rilevabile è  pressoché **//
//**    and is used as a seed for the random function.                    **    casuale e viene usato come seme per la funzione random.           **//
//** 3) A Rele card for Arduino with 4 relays. Purchasable on the         ** 3) Una scheda Rele per Arduino da 4 rele. Acquistabile  su internet. **//
//**    internet. The first three relays are associated with the buttons  **    I primi tre relè sono associati  ai pulsanti  visualizzati  nella **//
//**    displayed on the HTML page. The fourth is used to auto swith off, **    pagina HTML. Il quarto è  utilizzato  per autospegnersi, sia  per **//
//**    for security reasons (incorrect passwords, various timeouts) and  **    motivi di sicurezza (password errate, timeout vari) e sia per non **//
//**    also, to avoid waste of energy                                    **    sprecare inutilmente energia.                                     **//
//** ******************************************************************** ** ******************************************************************** **// 
//** This  sketch  is  distributed  under  the license of Postcardware or ** Questo sketch  è  distribuito  in  licenza  Postcardware / Cardware, **// 
//** cardware.  It  is  freely  usable  on moral  condition  to send me a ** liberamente  utilizzabile   a   condizione  morale  di spedirmi una  **// 
//** postcard (or an email) from your city  and make  mention of the main ** cartolina (o una  email)  dalla tua  città  e che citiate il sito e  **// 
//** website and the author in the comments of your program.              ** e l'autore nei commenti del vostro programma.                        **//
//** ******************************************************************** ** ******************************************************************** **// 

// added wifi shutdown for three incorrect password attempts              // aggiunto lo spegnimento wifi per tre tentativi di password errati
// Switch off after 2 minutes of inactivity                               // Spegnimento dopo 2 minuti di inattività
// Attempts to enter url artifacts, turn off WiFi.                         // Tentativi di immettere url a caso, spegne il WiFi. 

#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>

const char* ssid     = "WemosD1";
const char* password = "WemosD1Key";

IPAddress    apIP(1, 2, 3, 4);  

const int Pin_Rele        = 16;   //D2 - Relais
const int Pin_Cancello    = 5;    //D3 - Gate
const int Pin_Garage      = 4;    //D4 - Garage
const int Pin_Cancelletto = 14;   //D5 - Small Gate
const int Pin_Apertura    = 12;   //D6 - To comunicate to a alarm system when this Wemod D1 open a passage
                                  //     so, it can starts a timer for insert a password. 
                                  //     If the password is wrong the alarm system can activate alarm sirens. 

                                  
//Per prossima implementazione usa il pin 13 - D7 , nel caso collega 5, 4 e 14 ad un sommatore a diodi per avere la segnalazione 
//come quella sul pin_cancelletto senza ricorrere al pin 12 che quindi diventa libero. Poi se nnecessario più uscite usa il cmos 4051

// is better not use this pin
//   PIN      Accende    Means    
//   3        D0         RX  
//   1        D1         TX 

//   This are verified:
//   16       D2         IO                          Do not use, it go to 5V for a moment when you reset or turn on the Wemos D1 card. 
//   5        D3-D15     IO
//   4        D4-D14     IO     
//   14       D5-D13     IO-SCK
//   12       D6-D12     IO-MISO   
//   13       D7-D11     IO-MOSI
//   0        D8         IO-Pull up                  do not use, it go to 5V for a moment when you reset or turn on the Wemos D1 card
//   2        D9         IO-Pull up-Built_in led     do not use, it go to 5V for a moment when you reset or turn on the Wemos D1 card
//   15       D10        IO-Pull down,SS             do not use, it go to 5V for a moment when you reset or turn on the Wemos D1 card


ESP8266WebServer server(80);

String cookiesAutenticato ="";
String cookiesNonAutenticato ="";
String GarageCode ="";
String CancellettoCode ="";
String CancelloCode ="";
String SpegniWIFICode ="";

long oldMillisloop = 0;    //per il Serial.print dei messaggi 
long OldMillis = 0;        //inattivita utente
int  tentativi = 0; 



//** ******************************************************************** ** ******************************************************************** **// 
//** What kind of relay do we have?  If they get excited when the  pin of ** Che tipo di relè abbiamo? Se sono del tipo che si eccitano quando il **// 
//** the Wemos card goes  to 1, set TypeActivationRelay = 1;  if they are ** pin della scheda Wemos  va a 1, impostate  TipoAttivazioneRelay = 1; **// 
//** excited  when  the  pin  goes  to  0  set  TipoAttivazioneRelay = 0; ** Se si eccitano se il pin va a 0 impostate  TipoAttivazioneRelay = 0; **// 
//** ******************************************************************** ** ******************************************************************** **// 

int TipoAttivazioneRelay = 0;    
                                 
     


//Check if header is present and correct
bool is_authentified()
     {
     //Serial.println("Is_authentified");
     if (server.hasHeader("Cookie"))
        {
        String cookie = server.header("Cookie");
        if (cookie.indexOf(cookiesAutenticato) != -1) 
           {
           //Serial.println("Autenticato");
           return true;
           }
        }
     Serial.println("Non autenticato");
     return false;
     }

	 
//login page, also called for disconnect
void handleLogin(){
     Serial.println("----Enter handleLogin");
     OldMillis=millis();
     String msg;
     if (server.hasHeader("Cookie"))
        {
        Serial.print("Found cookie: ");
        String cookie = server.header("Cookie");
        Serial.println(cookie);
        }
 
     Serial.println("----handleLogin--1");       
     if (server.hasArg("DISCONNECT"))
        {
        Serial.println("Disconnection");
        server.sendHeader("Location","/login");
        server.sendHeader("Cache-Control","no-cache");
        server.sendHeader("Set-Cookie",cookiesNonAutenticato);
        server.send(301);
        return;
        }
     
     if (server.hasArg("MaggioriInfo"))
        {
        if (server.arg("MaggioriInfo") == "SI" )
           { 
           msg = "Inserisci utente e password...";
           Serial.println("Inserisci utente e password...");
           }
        }
		

	 
     if (server.hasArg("USERNAME") && server.hasArg("PASSWORD"))
        {
        Serial.print(server.arg("USERNAME"));
        Serial.print("  -*-  ");
        Serial.println(server.arg("PASSWORD"));
         
        if (server.arg("USERNAME") == "Admin" &&  server.arg("PASSWORD") == "12345" )
           {
           server.sendHeader("Location","/");
           server.sendHeader("Cache-Control","no-cache");
           server.sendHeader("Set-Cookie",cookiesAutenticato);
           server.send(301);
           Serial.println("Log in Successful");
           return;
           }
        msg = "Utente e password errate, prova ancora...";
        Serial.println("Log in Failed");
        tentativi=tentativi+1; Serial.print("tentativi=");Serial.println(tentativi);
        }

     Serial.println("----handleLogin--3");  
   
     String content =" ";
     content +="<style>";
     content +=".tabella {";
     content +="border:9px solid #FFFFFF;";
     content +="border-collapse:collapse;";
     content +="padding:5px;";
     content +="font-size:72;";
     content +="font-family:Trebuchet MS;";
     content +="font-weight:bold;";
     content +="}";
     content +="";
     content +=".tabella td {";
     content +="border:9px solid #FFFFFF;";
     content +="padding:5px;";
     content +="font-size:60;";
     content +="background:#80FFFF;}";
     content +="";
     content +=".myButton {";
     content +="-moz-box-shadow: 17px 15px 22px 9px #9fb4f2;";
     content +="-webkit-box-shadow: 17px 15px 22px 9px #9fb4f2;";
     content +="box-shadow: 17px 15px 22px 9px #9fb4f2;";
     content +="background:-webkit-gradient(linear, left top, left bottom, color-stop(0.05, #7892c2), color-stop(1, #476e9e));";
     content +="background:-moz-linear-gradient(top, #7892c2 5%, #476e9e 100%);";
     content +="background:-webkit-linear-gradient(top, #7892c2 5%, #476e9e 100%);";
     content +="background:-o-linear-gradient(top, #7892c2 5%, #476e9e 100%);";
     content +="background:-ms-linear-gradient(top, #7892c2 5%, #476e9e 100%);";
     content +="background:linear-gradient(to bottom, #7892c2 5%, #476e9e 100%);";
     content +="filter:progid:DXImageTransform.Microsoft.gradient(startColorstr='#7892c2', endColorstr='#476e9e',GradientType=0);";
     content +="background-color:#7892c2;";
     content +="-moz-border-radius:20px;";
     content +="-webkit-border-radius:20px;";
     content +="border-radius:20px;";
     content +="border:2px solid #4e6096;";
     content +="display:inline-block;";
     content +="cursor:pointer;";
     content +="color:#ffffff;";
     content +="font-family:Trebuchet MS;";
     content +="font-size:48px;";
     content +="font-weight:bold;";
     content +="padding:20px 40px;";
     content +="text-decoration:none;";
     content +="text-shadow:4px 4px 4px #283966;";
     content +="font-stretch:center;";
     content +="}";
     content +=".myButton:hover {";
     content +="background:-webkit-gradient(linear, left top, left bottom, color-stop(0.05, #476e9e), color-stop(1, #7892c2));";
     content +="background:-moz-linear-gradient(top, #476e9e 5%, #7892c2 100%);";
     content +="background:-webkit-linear-gradient(top, #476e9e 5%, #7892c2 100%);";
     content +="background:-o-linear-gradient(top, #476e9e 5%, #7892c2 100%);";
     content +="background:-ms-linear-gradient(top, #476e9e 5%, #7892c2 100%);";
     content +="background:linear-gradient(to bottom, #476e9e 5%, #7892c2 100%);";
     content +="filter:progid:DXImageTransform.Microsoft.gradient(startColorstr='#476e9e', endColorstr='#7892c2',GradientType=0);";
     content +="background-color:#476e9e;";
     content +="align='center';";
     content +="}";
     content +=".myButton:active {";
     content +="position:relative;";
     content +="top:1px;";
     content +="}";
     content +="";
     content +="";
     content +="</style>";
     content +="<style>input[type='text'] {font-size:96px; width: 550px; }</style>";
     content +="<style>input[type='submit'] {width: 5em; height: 3em; font-size: 96px;  width:440px ; Height:220px</style>";
     content +="<style>input[type='password'] { font-size: 96px; width: 550px; }</style>";
     content +="";
     content +="<html><body><form action='/login' method='POST'><br>";
     content +="";
     content +="<table class='tabella' align='center' >";
     content +="<caption><strong>Accedi al sistema<br>&nbsp;</caption>";
     content +="<thead>";
     content +="</thead>";
     content +="<tbody>";
     content +="<tr>&nbsp;</tr>";
     content +="<tr>";
     content +="<td><strong>Username..:</td>";
     content +="<td><input type='text' name='USERNAME' placeholder='user name' size='12' maxlength='12'></td>";
     content +="</tr>";
     content +="<tr>&nbsp;</tr>";
     content +="<tr>";
     content +="<td><strong>Password...:</td>";
     content +="<td><input type='password' name='PASSWORD' placeholder='password' size='12' maxlength='12'></td>";
     content +="</tr>";
     content +="";
     content +="</tbody>";
     content +="</table>";
     content +="<font size='22'>";
     content +="<br><br><div align ='center'>";
     content +="<input type='Submit' name='SUBMIT' class='myButton' value='Accedi'></form>";
     //content +="<font size'36'><br><br><br><br>"+ msg + "<br><br>";
     //content +="<font size'22'><strong><a href='/Azione?MaggioriInfo=SI'>Maggiori Informazioni</a></font></div></body></html>";
     
     content +="<font size'36'><br><br><br>";
     content +="<font size'22'><strong><a href='/Azione?MaggioriInfo=SI'>Maggiori Informazioni</a></font>";
     content +="<br><br><br><br>" + msg + "</div></body></html>";
     
     
     server.send(200, "text/html", content);
     delay(100);
     
     Serial.println("----END  handleLogin");
     }

//root page can be accessed only if authentification is ok
void handleRoot()
     {
     
     Serial.println("----Enter handleRoot");
     Serial.print("                             ");   
     Serial.println(server.arg("code").toInt());
  
     String header;
     if (!is_authentified())
        {
        server.sendHeader("Location","/login");
        server.sendHeader("Cache-Control","no-cache");
        server.send(301);
        return;
        }
      
     String content = "<html><body><H2>hello, you successfully connected to esp8266!</H2><br>";
     if (server.hasHeader("User-Agent"))
	      {
        content += "the user agent used is : " + server.header("User-Agent") + "<br><br>";
        }

     if (is_authentified())
        {
        if (server.hasArg("code"))
           {
           Serial.print( "-code------------------->") ;
           Serial.print( server.arg("code") );
            Serial.print( " ") ;
           if (server.arg("code") == GarageCode)      {Serial.println("GarageCode ");      ApriVarco(Pin_Garage);}
           if (server.arg("code") == CancelloCode)    {Serial.println("CancelloCode ");    ApriVarco(Pin_Cancello);}
           if (server.arg("code") == CancellettoCode) {Serial.println("CancellettoCode "); ApriVarco(Pin_Cancelletto);}
           if (server.arg("code") == SpegniWIFICode ) {Serial.println("SpegniWIFICode ");  SpegniWiFi();}
              
              
              server.sendHeader("Location","/");
              server.sendHeader("Cache-Control","no-cache");
              server.send(301);
              return;
           }

           // Build an HTML page to display on the web-server root address
           String content =" ";
           content +="<style>";
           content +=".myButton {height:280px; width:440px;";
           content +="box-shadow: 17px 15px 22px 9px #9fb4f2;";
           content +="background-color:#7892c2;";
           content +="-moz-border-radius:20px;";
           content +="-webkit-border-radius:20px;";
           content +="border:2px solid #4e6096;";
           content +="cursor:pointer;color:#ffffff;";
           content +="font-family:Trebuchet MS;font-size:54px;font-weight:bold;";
           content +="text-shadow:4px 4px 4px #283966;}";
           content +=".myButton:hover {background:-webkit-gradient(linear, left top, left bottom, color-stop(0.05, #476e9e), color-stop(1, #7892c2));}";
           content +=".myButton:active {position:relative; top:4px;left:4px;}";
           content +="</style>";
           content +="<table class=\"tabella\" align=\"center\" >";
           content +="<caption><font size=\"28\"><strong>Scegli il pulsante</font></caption>";
           content +="<tbody>";
           content +="<tr><td>&nbsp;</td></tr>";
           content +="<tr align=\"center\">";
           content +="<td>"; 
           content +="<form method=""post"" action=""code?code="+GarageCode+""">";
           content +="<button type=""submit"" class=""myButton"">Garage</button>";
           content +="</form>";
           content +="<td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>";
           content +="<td>"; 
           content +="<form method=""post"" action=""code?code="+CancelloCode+""">";
           content +="<button type=""submit"" class=""myButton"">Cancello</button>";
           content +="</form>";
           content +="</tr>";
           content +="<tr><td>&nbsp;</td></tr>";
           content +="<tr align=\"center\">";
           content +="<td>"; 
           content +="<form method=""post"" action=""code?code="+CancellettoCode+""">";
           content +="<button type=""submit"" class=""myButton"">Cancelletto</button>";
           content +="</form>";
           content +="<td></td>";
           content +="<td>"; 
           content +="<form method=""post"" action=""code?code="+SpegniWIFICode+">";
           content +="<button type=""submit"" class=""myButton"">Spegni WiFi</button>";
           content +="</form>";
           content +="</tr>";
           content +="</tbody>";
           content +="</table>";
           content +="<br><br>";
           content +="<div align=\"center\"><font size=\"6\"><strong>L\'uso non autorizzato e\' soggetto alle leggi vigenti.</div>";
           //content +="<br><div align=\"center\"><font size=\"6\"><strong>" + scrivi +  "</div>";
           server.send ( 200, "text/html", content );
       }

       //Serial.println("qua non ci devo passare");
       //  content += "You can access this page until you <a href=\"/login?DISCONNECT=YES\">disconnect</a></body></html>";
       //  server.send(200, "text/html", content);
     }

//no need authentification

void ApriVarco(int cosa)
     { 
           
     digitalWrite(cosa,TipoAttivazioneRelay); 
     digitalWrite(Pin_Apertura,TipoAttivazioneRelay); 
     delay(500); 
     digitalWrite(cosa,TipoAttivazioneRelay^1);  
     delay(500); 
     digitalWrite(Pin_Apertura,TipoAttivazioneRelay^1);
     OldMillis=millis();
     }


void handleNotFound(){
  Serial.println("----Enter handleNotFound");
  String message;
  message = "<html><body>";
  message +="<font size = '72'><strong><center>"; 
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET)?"GET":"POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  message +="<br><br><br><br>Url non permessa.";
  message +="</font>";
  message +="</body>";
  message +="</html>";
  Serial.println(message);
  server.send ( 200, "text/html", message );
  delay(100);
  SpegniWiFi(); 
 
}

void setup(void)
     {
     
     Imposta_i_pin(); 

     Serial.begin(115200);
     delay(500);
     Serial.println("CorrOpenGateWifi - Version: 1.8.2");
     Serial.println("  Written By Corrado Bevilacqua");
     
     //set-up the custom IP address
     WiFi.mode(WIFI_AP_STA);
     WiFi.softAPConfig(apIP, apIP, IPAddress(255, 255, 255, 0));   // subnet FF FF FF 00 
 
     /* You can remove the password parameter if you want the AP to be open. */
     WiFi.softAP(ssid, password);

     IPAddress myIP = WiFi.softAPIP();
     Serial.print("AP IP address: ");
     Serial.println(myIP);


     //Mettere un filo come antenna per generare numeri casuali
     //per aumentare la sicurezza del cookies
     long somma =0;
     for (int i=0; i<=100; i++)
         {
         somma+=analogRead(0);
         //Serial.println(somma);
         //delay(5);
         }

     Serial.print("Seme random = ");
     Serial.println(somma);

     randomSeed(somma);

     cookiesNonAutenticato ="0";

     cookiesAutenticato=random(65,120);cookiesAutenticato+=random(65,120);cookiesAutenticato+=random(65,120);cookiesAutenticato+=random(65,120);
     Serial.println(cookiesAutenticato);

     GarageCode = random(65,120); GarageCode +=random(65,120); GarageCode +=random(65,120); GarageCode +=random(65,120);     
     Serial.println("GarageCode=");Serial.println(GarageCode);

     CancellettoCode = random(65,120);
     CancellettoCode += random(65,120);
     CancellettoCode += random(65,120);
     CancellettoCode += random(65,120);
     Serial.println("CancellettoCode=");Serial.println(CancellettoCode);

     CancelloCode = random(65,120);
     CancelloCode += random(65,120); CancelloCode += random(65,120);CancelloCode += random(65,120);
     Serial.println("CancelloCode=");Serial.println(CancelloCode);

     SpegniWIFICode = random(65,120);
     SpegniWIFICode += random(65,120);SpegniWIFICode += random(65,120);SpegniWIFICode += random(65,120);
     Serial.println("SpegniWIFICode=");Serial.println(SpegniWIFICode);

  
     server.on("/", handleRoot);
     server.on("/login", handleLogin);
     server.on ( "/code", handleRoot);
     server.on("/favicon.ico", handlefavicon);
     server.on ( "/Azione", handleLogin);        
     server.on("/inline", []() {server.send(200, "text/plain", "this works without need of authentification");}	);

     server.onNotFound(handleNotFound);
     //here the list of headers to be recorded
     const char * headerkeys[] = {"User-Agent","Cookie"} ;
     size_t headerkeyssize = sizeof(headerkeys)/sizeof(char*);
     //ask server to track these headers
     server.collectHeaders(headerkeys, headerkeyssize );
     server.begin();
     Serial.println("HTTP server started");

     OldMillis = millis();
     oldMillisloop= millis();
  
     }

void loop(void)
     {
     server.handleClient();
     delay(5);

     if (millis()-oldMillisloop > 2500)
        {
        Serial.print("Tempo=");  
        Serial.print(millis()); 
          
        Serial.print("    Inattivita=");  
        Serial.print(millis()-OldMillis); 
        Serial.print("    tentativi=");  
        Serial.println(tentativi); 
        oldMillisloop=millis();
        }
     
     //120 secondi di inattività per spegnere il wifi
     if (millis()-OldMillis > 120000) { SpegniWiFi(); }
             
     //300 secondi (5 minuti) dall'accensione spegni il wifi
     if (millis() > 300000)           { SpegniWiFi(); }
          
     //Tre password sbagliate per spegnere il wifi
     if (tentativi >= 3)              { SpegniWiFi(); }
     }


void SpegniWiFi() 
     {
     delay(2000);
     Serial.println("Wifi off");

     WiFi.disconnect();
     WiFi.mode(WIFI_OFF);
     //  WiFi.ForceSleeping();
     delay(10); 
     for (int i=1; i<=10; i++)
         {
         delay(1000);
         Serial.print("."); 
         }
         
     digitalWrite(Pin_Rele,TipoAttivazioneRelay^1);   //tolgo anche l'alimentazione al relè in modo che è completamente spento  

     //Arrivo qua solo se non sono collegato con il rele che toglie la corrente e quindi giro qua per sempre 
     while (1==1) { delay(10000); }
          
     }   
void handlefavicon()
     {  
     Serial.println("favicon");
     }


void Imposta_i_pin()
     {

//   pinMode(16,OUTPUT); digitalWrite(16,TipoAttivazioneRelay);  //spegni tutti i pin tranne il D2 al quale attaccato il relè per autospegnersi
//   pinMode(5,OUTPUT);  digitalWrite(5,TipoAttivazioneRelay^1);
//   pinMode(4,OUTPUT);  digitalWrite(4,TipoAttivazioneRelay^1);
//   pinMode(14,OUTPUT); digitalWrite(14,TipoAttivazioneRelay^1);
//   pinMode(12,OUTPUT); digitalWrite(12,TipoAttivazioneRelay^1);
//   pinMode(13,OUTPUT); digitalWrite(13,TipoAttivazioneRelay^1);
//   pinMode(0,OUTPUT);  digitalWrite(0,TipoAttivazioneRelay^1);
//   pinMode(2,OUTPUT);  digitalWrite(2,TipoAttivazioneRelay^1);
//   pinMode(15,OUTPUT); digitalWrite(15,TipoAttivazioneRelay^1);

     



     pinMode(Pin_Rele,OUTPUT);         digitalWrite(Pin_Rele,TipoAttivazioneRelay);        // PIN 16 - D2  spegni tutti i tranne questo al quale attaccato il relè per autospegnersi
     pinMode(Pin_Cancello,OUTPUT);     digitalWrite(Pin_Cancello,TipoAttivazioneRelay^1);    // PIN 5 - D3 
     pinMode(Pin_Garage,OUTPUT);       digitalWrite(Pin_Garage,TipoAttivazioneRelay^1);      // PIN 4 - D4
     pinMode(Pin_Cancelletto,OUTPUT);  digitalWrite(Pin_Cancelletto,TipoAttivazioneRelay^1); // PIN 14 - D5
     pinMode(Pin_Apertura,OUTPUT);     digitalWrite(Pin_Apertura,TipoAttivazioneRelay^1);    // PIN 12 - D6
     pinMode(13,OUTPUT);               digitalWrite(13,TipoAttivazioneRelay^1);
     pinMode(0,OUTPUT);                digitalWrite(0,TipoAttivazioneRelay^1);
     pinMode(2,OUTPUT);                digitalWrite(2,TipoAttivazioneRelay^1);
     pinMode(15,OUTPUT);               digitalWrite(15,TipoAttivazioneRelay^1);

     }
      

     

